@@ -7,15 +7,15 @@ use std::{
     str,
 };
 
-use ansi_term::Colour;
+use anstyle::{AnsiColor, Color, Style};
 use anyhow::{anyhow, Context, Result};
-use difference::{Changeset, Difference};
 use indoc::indoc;
 use lazy_static::lazy_static;
 use regex::{
     bytes::{Regex as ByteRegex, RegexBuilder as ByteRegexBuilder},
     Regex,
 };
+use similar::{ChangeTag, TextDiff};
 use tree_sitter::{format_sexp, Language, LogType, Parser, Query};
 use walkdir::WalkDir;
 
@@ -27,7 +27,7 @@ lazy_static! {
            (?P<equals>(?:=+){3,})
            (?P<suffix1>[^=\r\n][^\r\n]*)?
            \r?\n
-           (?P<test_name_and_markers>(?:[^=\r\n][^\r\n]*\r?\n)+)
+           (?P<test_name_and_markers>(?:([^=\r\n]|\s+:)[^\r\n]*\r?\n)+)
            ===+
            (?P<suffix2>[^=\r\n][^\r\n]*)?\r?\n"
     )
@@ -59,6 +59,7 @@ pub enum TestEntry {
         header_delim_len: usize,
         divider_delim_len: usize,
         has_fields: bool,
+        attributes_str: String,
         attributes: TestAttributes,
     },
 }
@@ -98,12 +99,15 @@ pub struct TestOptions<'a> {
     pub path: PathBuf,
     pub debug: bool,
     pub debug_graph: bool,
-    pub filter: Option<&'a str>,
     pub include: Option<Regex>,
     pub exclude: Option<Regex>,
     pub update: bool,
     pub open_log: bool,
     pub languages: BTreeMap<&'a str, &'a Language>,
+    pub color: bool,
+    pub test_num: usize,
+    pub show_fields: bool,
+    pub overview_only: bool,
 }
 
 pub fn run_tests_at_path(parser: &mut Parser, opts: &mut TestOptions) -> Result<()> {
@@ -156,20 +160,36 @@ pub fn run_tests_at_path(parser: &mut Parser, opts: &mut TestOptions) -> Result<
         } else {
             has_parse_errors = opts.update && has_parse_errors;
 
-            if !has_parse_errors {
-                if failures.len() == 1 {
-                    println!("1 failure:");
-                } else {
-                    println!("{} failures:", failures.len());
+            if !opts.overview_only {
+                if !has_parse_errors {
+                    if failures.len() == 1 {
+                        println!("1 failure:");
+                    } else {
+                        println!("{} failures:", failures.len());
+                    }
                 }
-            }
 
-            print_diff_key();
-            for (i, (name, actual, expected)) in failures.iter().enumerate() {
-                println!("\n  {}. {name}:", i + 1);
-                let actual = format_sexp(actual, 2);
-                let expected = format_sexp(expected, 2);
-                print_diff(&actual, &expected);
+                if opts.color {
+                    print_diff_key();
+                }
+                for (i, (name, actual, expected)) in failures.iter().enumerate() {
+                    if expected == "NO ERROR" {
+                        println!("\n  {}. {name}:\n", i + 1);
+                        println!("  Expected an ERROR node, but got:");
+                        println!(
+                            "  {}",
+                            paint(
+                                opts.color.then_some(AnsiColor::Red),
+                                &format_sexp(actual, 2)
+                            )
+                        );
+                    } else {
+                        println!("\n  {}. {name}:", i + 1);
+                        let actual = format_sexp(actual, 2);
+                        let expected = format_sexp(expected, 2);
+                        print_diff(&actual, &expected, opts.color);
+                    }
+                }
             }
 
             if has_parse_errors {
@@ -183,6 +203,64 @@ pub fn run_tests_at_path(parser: &mut Parser, opts: &mut TestOptions) -> Result<
     }
 }
 
+#[allow(clippy::type_complexity)]
+pub fn get_test_info<'test>(
+    test_entry: &'test TestEntry,
+    target_test: u32,
+    test_num: &mut u32,
+) -> Option<(&'test str, &'test [u8], Vec<Box<str>>)> {
+    match test_entry {
+        TestEntry::Example {
+            name,
+            input,
+            attributes,
+            ..
+        } => {
+            if *test_num == target_test {
+                return Some((name, input, attributes.languages.clone()));
+            }
+            *test_num += 1;
+        }
+        TestEntry::Group { children, .. } => {
+            for child in children {
+                if let Some((name, input, languages)) = get_test_info(child, target_test, test_num)
+                {
+                    return Some((name, input, languages));
+                }
+            }
+        }
+    }
+
+    None
+}
+
+/// Writes the input of `target_test` to a temporary file and returns the path
+pub fn get_tmp_test_file(target_test: u32, color: bool) -> Result<(PathBuf, Vec<Box<str>>)> {
+    let current_dir = std::env::current_dir().unwrap();
+    let test_dir = current_dir.join("test").join("corpus");
+
+    // Get the input of the target test
+    let test_entry = parse_tests(&test_dir)?;
+    let mut test_num = 0;
+    let Some((test_name, test_contents, languages)) =
+        get_test_info(&test_entry, target_test - 1, &mut test_num)
+    else {
+        return Err(anyhow!("Failed to fetch contents of test #{target_test}"));
+    };
+
+    // Write the test contents to a temporary file
+    let test_path = std::env::temp_dir().join(".tree-sitter-test");
+    let mut test_file = std::fs::File::create(&test_path)?;
+    test_file.write_all(test_contents)?;
+
+    println!(
+        "{target_test}. {}\n",
+        paint(color.then_some(AnsiColor::Green), test_name)
+    );
+
+    Ok((test_path, languages))
+}
+
 pub fn check_queries_at_path(language: &Language, path: &Path) -> Result<()> {
     if path.exists() {
         for entry in WalkDir::new(path)
@@ -207,36 +285,62 @@ pub fn check_queries_at_path(language: &Language, path: &Path) -> Result<()> {
 pub fn print_diff_key() {
     println!(
         "\ncorrect / {} / {}",
-        Colour::Green.paint("expected"),
-        Colour::Red.paint("unexpected")
+        paint(Some(AnsiColor::Green), "expected"),
+        paint(Some(AnsiColor::Red), "unexpected")
     );
 }
 
-pub fn print_diff(actual: &str, expected: &str) {
-    let changeset = Changeset::new(actual, expected, "\n");
-    for diff in &changeset.diffs {
-        match diff {
-            Difference::Same(part) => {
-                print!("{part}{}", changeset.split);
+pub fn print_diff(actual: &str, expected: &str, use_color: bool) {
+    let diff = TextDiff::from_lines(actual, expected);
+    for diff in diff.iter_all_changes() {
+        match diff.tag() {
+            ChangeTag::Equal => {
+                if use_color {
+                    print!("{diff}");
+                } else {
+                    print!(" {diff}");
+                }
             }
-            Difference::Add(part) => {
-                print!("{}{}", Colour::Green.paint(part), changeset.split);
+            ChangeTag::Insert => {
+                if use_color {
+                    print!("{}", paint(Some(AnsiColor::Green), diff.as_str().unwrap()));
+                } else {
+                    print!("+{diff}");
+                }
+                if diff.missing_newline() {
+                    println!();
+                }
             }
-            Difference::Rem(part) => {
-                print!("{}{}", Colour::Red.paint(part), changeset.split);
+            ChangeTag::Delete => {
+                if use_color {
+                    print!("{}", paint(Some(AnsiColor::Red), diff.as_str().unwrap()));
+                } else {
+                    print!("-{diff}");
+                }
+                if diff.missing_newline() {
+                    println!();
+                }
             }
         }
     }
+
     println!();
 }
 
+pub fn paint(color: Option<AnsiColor>, text: &str) -> String {
+    let style = Style::new().fg_color(color.map(Color::Ansi));
+    format!("{style}{text}{style:#}")
+}
+
+/// This will return false if we want to "fail fast". It will bail and not parse any more tests.
+#[allow(clippy::too_many_arguments)]
 fn run_tests(
     parser: &mut Parser,
     test_entry: TestEntry,
     opts: &mut TestOptions,
     mut indent_level: i32,
     failures: &mut Vec<(String, String, String)>,
-    corrected_entries: &mut Vec<(String, String, String, usize, usize)>,
+    corrected_entries: &mut Vec<(String, String, String, String, usize, usize)>,
     has_parse_errors: &mut bool,
 ) -> Result<bool> {
     match test_entry {
@@ -247,17 +351,26 @@ fn run_tests(
             header_delim_len,
             divider_delim_len,
             has_fields,
+            attributes_str,
             attributes,
         } => {
             print!("{}", "  ".repeat(indent_level as usize));
 
             if attributes.skip {
-                println!(" {}", Colour::Yellow.paint(&name));
+                println!(
+                    "{:>3}.  {}",
+                    opts.test_num,
+                    paint(opts.color.then_some(AnsiColor::Yellow), &name),
+                );
                 return Ok(true);
             }
 
             if !attributes.platform {
-                println!(" {}", Colour::Purple.paint(&name));
+                println!(
+                    "{:>3}.  {}",
+                    opts.test_num,
+                    paint(opts.color.then_some(AnsiColor::Magenta), &name),
+                );
                 return Ok(true);
             }
 
@@ -273,22 +386,11 @@ fn run_tests(
 
                 if attributes.error {
                     if tree.root_node().has_error() {
-                        println!(" {}", Colour::Green.paint(&name));
-                    } else {
-                        println!(" {}", Colour::Red.paint(&name));
-                    }
-
-                    if attributes.fail_fast {
-                        return Ok(false);
-                    }
-                } else {
-                    let mut actual = tree.root_node().to_sexp();
-                    if !has_fields {
-                        actual = strip_sexp_fields(&actual);
-                    }
-
-                    if actual == output {
-                        println!("✓ {}", Colour::Green.paint(&name));
+                        println!(
+                            "{:>3}.  {}",
+                            opts.test_num,
+                            paint(opts.color.then_some(AnsiColor::Green), &name)
+                        );
                         if opts.update {
                             let input = String::from_utf8(input.clone()).unwrap();
                             let output = format_sexp(&output, 0);
@@ -296,6 +398,60 @@ fn run_tests(
                                 name.clone(),
                                 input,
                                 output,
+                                attributes_str.clone(),
+                                header_delim_len,
+                                divider_delim_len,
+                            ));
+                        }
+                    } else {
+                        if opts.update {
+                            let input = String::from_utf8(input.clone()).unwrap();
+                            // Keep the original `expected` output if the actual output has no error
+                            let output = format_sexp(&output, 0);
+                            corrected_entries.push((
+                                name.clone(),
+                                input,
+                                output,
+                                attributes_str.clone(),
+                                header_delim_len,
+                                divider_delim_len,
+                            ));
+                        }
+                        println!(
+                            "{:>3}.  {}",
+                            opts.test_num,
+                            paint(opts.color.then_some(AnsiColor::Red), &name)
+                        );
+                        failures.push((
+                            name.clone(),
+                            tree.root_node().to_sexp(),
+                            "NO ERROR".to_string(),
+                        ));
+                    }
+
+                    if attributes.fail_fast {
+                        return Ok(false);
+                    }
+                } else {
+                    let mut actual = tree.root_node().to_sexp();
+                    if !(opts.show_fields || has_fields) {
+                        actual = strip_sexp_fields(&actual);
+                    }
+
+                    if actual == output {
+                        println!(
+                            "{:>3}. ✓ {}",
+                            opts.test_num,
+                            paint(opts.color.then_some(AnsiColor::Green), &name)
+                        );
+                        if opts.update {
+                            let input = String::from_utf8(input.clone()).unwrap();
+                            let output = format_sexp(&output, 0);
+                            corrected_entries.push((
+                                name.clone(),
+                                input,
+                                output,
+                                attributes_str.clone(),
                                 header_delim_len,
                                 divider_delim_len,
                             ));
@@ -319,6 +475,7 @@ fn run_tests(
                                     name.clone(),
                                     input,
                                     expected_output,
+                                    attributes_str.clone(),
                                     header_delim_len,
                                     divider_delim_len,
                                 ));
@@ -327,68 +484,108 @@ fn run_tests(
                                     name.clone(),
                                     input,
                                     actual_output,
+                                    attributes_str.clone(),
                                     header_delim_len,
                                     divider_delim_len,
                                 ));
-                                println!("✓ {}", Colour::Blue.paint(&name));
+                                println!(
+                                    "{:>3}. ✓ {}",
+                                    opts.test_num,
+                                    paint(opts.color.then_some(AnsiColor::Blue), &name),
+                                );
                             }
                         } else {
-                            println!("✗ {}", Colour::Red.paint(&name));
+                            println!(
+                                "{:>3}. ✗ {}",
+                                opts.test_num,
+                                paint(opts.color.then_some(AnsiColor::Red), &name),
+                            );
                         }
                         failures.push((name.clone(), actual, output.clone()));
 
                         if attributes.fail_fast {
-                            // return value of false means to fail fast
                             return Ok(false);
                         }
-
-                        if i == attributes.languages.len() - 1 {
-                            // reset back to first language
-                            parser.set_language(opts.languages.values().next().unwrap())?;
-                        }
                     }
                 }
+
+                if i == attributes.languages.len() - 1 {
+                    // reset to the first language
+                    parser.set_language(opts.languages.values().next().unwrap())?;
+                }
             }
+            opts.test_num += 1;
         }
         TestEntry::Group {
             name,
-            mut children,
+            children,
             file_path,
         } => {
-            children.retain(|child| {
-                if let TestEntry::Example { name, .. } = child {
-                    if let Some(filter) = opts.filter {
-                        if !name.contains(filter) {
-                            return false;
-                        }
-                    }
-                    if let Some(include) = &opts.include {
-                        if !include.is_match(name) {
-                            return false;
-                        }
-                    }
-                    if let Some(exclude) = &opts.exclude {
-                        if exclude.is_match(name) {
-                            return false;
-                        }
-                    }
-                }
-                true
-            });
-
             if children.is_empty() {
                 return Ok(true);
             }
 
-            if indent_level > 0 {
-                print!("{}", "  ".repeat(indent_level as usize));
-                println!("{name}:");
-            }
-
-            let failure_count = failures.len();
-
             indent_level += 1;
+            let mut advance_counter = opts.test_num;
+            let failure_count = failures.len();
+            let mut has_printed = false;
+            let mut skipped_tests = 0;
+
+            let matches_filter = |name: &str, opts: &TestOptions| {
+                if let Some(include) = &opts.include {
+                    include.is_match(name)
+                } else if let Some(exclude) = &opts.exclude {
+                    !exclude.is_match(name)
+                } else {
+                    true
+                }
+            };
+
+            let mut should_skip = |entry: &TestEntry, opts: &TestOptions| match entry {
+                TestEntry::Example { name, .. } => {
+                    advance_counter += 1;
+                    !matches_filter(name, opts)
+                }
+                TestEntry::Group { .. } => {
+                    advance_counter += count_subtests(entry);
+                    false
+                }
+            };
+
             for child in children {
+                if let TestEntry::Example {
+                    ref name,
+                    ref input,
+                    ref output,
+                    ref attributes_str,
+                    header_delim_len,
+                    divider_delim_len,
+                    ..
+                } = child
+                {
+                    if should_skip(&child, opts) {
+                        let input = String::from_utf8(input.clone()).unwrap();
+                        let output = format_sexp(output, 0);
+                        corrected_entries.push((
+                            name.clone(),
+                            input,
+                            output,
+                            attributes_str.clone(),
+                            header_delim_len,
+                            divider_delim_len,
+                        ));
+
+                        opts.test_num += 1;
+                        skipped_tests += 1;
+
+                        continue;
+                    }
+                }
+                if !has_printed && indent_level > 1 {
+                    has_printed = true;
+                    print!("{}", "  ".repeat((indent_level - 1) as usize));
+                    println!("{name}:");
+                }
                 if !run_tests(
                     parser,
                     child,
@@ -403,6 +600,8 @@ fn run_tests(
                 }
             }
 
+            opts.test_num += skipped_tests;
+
             if let Some(file_path) = file_path {
                 if opts.update && failures.len() - failure_count > 0 {
                     write_tests(&file_path, corrected_entries)?;
@@ -414,9 +613,18 @@ fn run_tests(
     Ok(true)
 }
 
+fn count_subtests(test_entry: &TestEntry) -> usize {
+    match test_entry {
+        TestEntry::Example { .. } => 1,
+        TestEntry::Group { children, .. } => children
+            .iter()
+            .fold(0, |count, child| count + count_subtests(child)),
+    }
+}
+
 fn write_tests(
     file_path: &Path,
-    corrected_entries: &[(String, String, String, usize, usize)],
+    corrected_entries: &[(String, String, String, String, usize, usize)],
 ) -> Result<()> {
     let mut buffer = fs::File::create(file_path)?;
     write_tests_to_buffer(&mut buffer, corrected_entries)
@@ -424,9 +632,9 @@ fn write_tests(
 
 fn write_tests_to_buffer(
     buffer: &mut impl Write,
-    corrected_entries: &[(String, String, String, usize, usize)],
+    corrected_entries: &[(String, String, String, String, usize, usize)],
 ) -> Result<()> {
-    for (i, (name, input, output, header_delim_len, divider_delim_len)) in
+    for (i, (name, input, output, attributes_str, header_delim_len, divider_delim_len)) in
         corrected_entries.iter().enumerate()
     {
         if i > 0 {
@@ -434,8 +642,13 @@ fn write_tests_to_buffer(
         }
         writeln!(
             buffer,
-            "{}\n{name}\n{}\n{input}\n{}\n\n{}",
+            "{}\n{name}\n{}{}\n{input}\n{}\n\n{}",
             "=".repeat(*header_delim_len),
+            if attributes_str.is_empty() {
+                attributes_str.clone()
+            } else {
+                format!("{attributes_str}\n")
+            },
             "=".repeat(*header_delim_len),
             "-".repeat(*divider_delim_len),
             output.trim()
@@ -493,6 +706,7 @@ fn parse_test_content(name: String, content: &str, file_path: Option<PathBuf>) -
     let mut children = Vec::new();
     let bytes = content.as_bytes();
     let mut prev_name = String::new();
+    let mut prev_attributes_str = String::new();
     let mut prev_header_end = 0;
 
     // Find the first test header in the file, and determine if it has a
@@ -523,17 +737,20 @@ fn parse_test_content(name: String, content: &str, file_path: Option<PathBuf>) -
             .map_or("".as_bytes(), |m| m.as_bytes());
 
         let mut test_name = String::new();
+        let mut attributes_str = String::new();
+
         let mut seen_marker = false;
 
+        let test_name_and_markers = str::from_utf8(test_name_and_markers).unwrap();
         for line in test_name_and_markers
-            .split(|&c| c == b'\n')
+            .split_inclusive('\n')
             .filter(|s| !s.is_empty())
         {
-            let line = str::from_utf8(line).unwrap();
-            match line.split('(').next().unwrap() {
+            let trimmed_line = line.trim();
+            match trimmed_line.split('(').next().unwrap() {
                 ":skip" => (seen_marker, skip) = (true, true),
                 ":platform" => {
-                    if let Some(platforms) = line.strip_prefix(':').and_then(|s| {
+                    if let Some(platforms) = trimmed_line.strip_prefix(':').and_then(|s| {
                         s.strip_prefix("platform(")
                             .and_then(|s| s.strip_suffix(')'))
                     }) {
@@ -546,7 +763,7 @@ fn parse_test_content(name: String, content: &str, file_path: Option<PathBuf>) -
                 ":fail-fast" => (seen_marker, fail_fast) = (true, true),
                 ":error" => (seen_marker, error) = (true, true),
                 ":language" => {
-                    if let Some(lang) = line.strip_prefix(':').and_then(|s| {
+                    if let Some(lang) = trimmed_line.strip_prefix(':').and_then(|s| {
                         s.strip_prefix("language(")
                             .and_then(|s| s.strip_suffix(')'))
                     }) {
@@ -556,11 +773,11 @@ fn parse_test_content(name: String, content: &str, file_path: Option<PathBuf>) -
                 }
                 _ if !seen_marker => {
                     test_name.push_str(line);
-                    test_name.push('\n');
                 }
                 _ => {}
             }
         }
+        attributes_str.push_str(test_name_and_markers.strip_prefix(&test_name).unwrap());
 
         // prefer skip over error, both shouldn't be set
         if skip {
@@ -579,10 +796,16 @@ fn parse_test_content(name: String, content: &str, file_path: Option<PathBuf>) -
             } else {
                 Some(test_name.trim_end().to_string())
             };
+            let attributes_str = if attributes_str.is_empty() {
+                None
+            } else {
+                Some(attributes_str.trim_end().to_string())
+            };
             Some((
                 header_delim_len,
                 header_range,
                 test_name,
+                attributes_str,
                 TestAttributes {
                     skip,
                     platform: platform.unwrap_or(true),
@@ -597,12 +820,15 @@ fn parse_test_content(name: String, content: &str, file_path: Option<PathBuf>) -
     });
 
     let (mut prev_header_len, mut prev_attributes) = (80, TestAttributes::default());
-    for (header_delim_len, header_range, test_name, attributes) in header_matches.chain(Some((
-        80,
-        bytes.len()..bytes.len(),
-        None,
-        TestAttributes::default(),
-    ))) {
+    for (header_delim_len, header_range, test_name, attributes_str, attributes) in header_matches
+        .chain(Some((
+            80,
+            bytes.len()..bytes.len(),
+            None,
+            None,
+            TestAttributes::default(),
+        )))
+    {
         // Find the longest line of dashes following each test description. That line
         // separates the input from the expected output. Ignore any matches whose suffix
         // does not match the first suffix in the file.
@@ -654,6 +880,7 @@ fn parse_test_content(name: String, content: &str, file_path: Option<PathBuf>) -
                         header_delim_len: prev_header_len,
                         divider_delim_len,
                         has_fields,
+                        attributes_str: prev_attributes_str,
                         attributes: prev_attributes,
                     };
 
@@ -663,6 +890,7 @@ fn parse_test_content(name: String, content: &str, file_path: Option<PathBuf>) -
         }
         prev_attributes = attributes;
         prev_name = test_name.unwrap_or_default();
+        prev_attributes_str = attributes_str.unwrap_or_default();
         prev_header_len = header_delim_len;
         prev_header_end = header_range.end;
     }
@@ -716,6 +944,7 @@ d
                         header_delim_len: 15,
                         divider_delim_len: 3,
                         has_fields: false,
+                        attributes_str: String::new(),
                         attributes: TestAttributes::default(),
                     },
                     TestEntry::Example {
@@ -725,6 +954,7 @@ d
                         header_delim_len: 16,
                         divider_delim_len: 3,
                         has_fields: false,
+                        attributes_str: String::new(),
                         attributes: TestAttributes::default(),
                     },
                 ],
@@ -775,6 +1005,7 @@ abc
                         header_delim_len: 18,
                         divider_delim_len: 7,
                         has_fields: false,
+                        attributes_str: String::new(),
                         attributes: TestAttributes::default(),
                     },
                     TestEntry::Example {
@@ -784,6 +1015,7 @@ abc
                         header_delim_len: 25,
                         divider_delim_len: 19,
                         has_fields: false,
+                        attributes_str: String::new(),
                         attributes: TestAttributes::default(),
                     },
                 ],
@@ -831,12 +1063,12 @@ abc
                 r"(source_file (ERROR (UNEXPECTED 'f') (UNEXPECTED '+')))",
                 0
             ),
-            r#"
+            r"
 (source_file
   (ERROR
     (UNEXPECTED 'f')
     (UNEXPECTED '+')))
-"#
+"
             .trim()
         );
     }
@@ -849,6 +1081,7 @@ abc
                 "title 1".to_string(),
                 "input 1".to_string(),
                 "output 1".to_string(),
+                String::new(),
                 80,
                 80,
             ),
@@ -856,6 +1089,7 @@ abc
                 "title 2".to_string(),
                 "input 2".to_string(),
                 "output 2".to_string(),
+                String::new(),
                 80,
                 80,
             ),
@@ -936,6 +1170,7 @@ code
                         header_delim_len: 18,
                         divider_delim_len: 3,
                         has_fields: false,
+                        attributes_str: String::new(),
                         attributes: TestAttributes::default(),
                     },
                     TestEntry::Example {
@@ -945,6 +1180,7 @@ code
                         header_delim_len: 18,
                         divider_delim_len: 3,
                         has_fields: false,
+                        attributes_str: String::new(),
                         attributes: TestAttributes::default(),
                     },
                     TestEntry::Example {
@@ -954,6 +1190,7 @@ code
                         header_delim_len: 25,
                         divider_delim_len: 3,
                         has_fields: false,
+                        attributes_str: String::new(),
                         attributes: TestAttributes::default(),
                     }
                 ],
@@ -1005,7 +1242,27 @@ NOT A TEST HEADER
 ---asdf\()[]|{}*+?^$.-
 
 (a)
-        "
+
+==============================asdf\()[]|{}*+?^$.-
+Test containing equals
+==============================asdf\()[]|{}*+?^$.-
+
+===
+
+------------------------------asdf\()[]|{}*+?^$.-
+
+(a)
+
+==============================asdf\()[]|{}*+?^$.-
+Subsequent test containing equals
+==============================asdf\()[]|{}*+?^$.-
+
+===
+
+------------------------------asdf\()[]|{}*+?^$.-
+
+(a)
+"
             .trim(),
             None,
         );
@@ -1015,7 +1272,7 @@ NOT A TEST HEADER
             =========================\n\
             -------------------------\n"
             .to_vec();
-        assert_eq!(
+        pretty_assertions::assert_eq!(
             entry,
             TestEntry::Group {
                 name: "the-filename".to_string(),
@@ -1027,6 +1284,7 @@ NOT A TEST HEADER
                         header_delim_len: 18,
                         divider_delim_len: 3,
                         has_fields: false,
+                        attributes_str: String::new(),
                         attributes: TestAttributes::default(),
                     },
                     TestEntry::Example {
@@ -1036,6 +1294,7 @@ NOT A TEST HEADER
                         header_delim_len: 18,
                         divider_delim_len: 3,
                         has_fields: false,
+                        attributes_str: String::new(),
                         attributes: TestAttributes::default(),
                     },
                     TestEntry::Example {
@@ -1045,6 +1304,27 @@ NOT A TEST HEADER
                         header_delim_len: 25,
                         divider_delim_len: 3,
                         has_fields: false,
+                        attributes_str: String::new(),
+                        attributes: TestAttributes::default(),
+                    },
+                    TestEntry::Example {
+                        name: "Test containing equals".to_string(),
+                        input: "\n===\n".into(),
+                        output: "(a)".into(),
+                        header_delim_len: 30,
+                        divider_delim_len: 30,
+                        has_fields: false,
+                        attributes_str: String::new(),
+                        attributes: TestAttributes::default(),
+                    },
+                    TestEntry::Example {
+                        name: "Subsequent test containing equals".to_string(),
+                        input: "\n===\n".into(),
+                        output: "(a)".into(),
+                        header_delim_len: 30,
+                        divider_delim_len: 30,
+                        has_fields: false,
+                        attributes_str: String::new(),
                         attributes: TestAttributes::default(),
                     }
                 ],
@@ -1090,6 +1370,7 @@ code with ----
                         header_delim_len: 15,
                         divider_delim_len: 3,
                         has_fields: false,
+                        attributes_str: String::new(),
                         attributes: TestAttributes::default(),
                     },
                     TestEntry::Example {
@@ -1099,6 +1380,7 @@ code with ----
                         header_delim_len: 20,
                         divider_delim_len: 3,
                         has_fields: false,
+                        attributes_str: String::new(),
                         attributes: TestAttributes::default(),
                     }
                 ]
@@ -1136,6 +1418,7 @@ a
                     header_delim_len: 21,
                     divider_delim_len: 3,
                     has_fields: false,
+                    attributes_str: ":skip".to_string(),
                     attributes: TestAttributes {
                         skip: true,
                         platform: true,
@@ -1163,6 +1446,7 @@ a
 =============================
 Test with bad platform marker
 :platform({})
+
 :language(foo)
 =============================
 a
@@ -1192,6 +1476,7 @@ a
                         header_delim_len: 25,
                         divider_delim_len: 3,
                         has_fields: false,
+                        attributes_str: format!(":platform({})\n:fail-fast", std::env::consts::OS),
                         attributes: TestAttributes {
                             skip: false,
                             platform: true,
@@ -1207,6 +1492,11 @@ a
                         header_delim_len: 29,
                         divider_delim_len: 3,
                         has_fields: false,
+                        attributes_str: if std::env::consts::OS == "linux" {
+                            ":platform(macos)\n\n:language(foo)".to_string()
+                        } else {
+                            ":platform(linux)\n\n:language(foo)".to_string()
+                        },
                         attributes: TestAttributes {
                             skip: false,
                             platform: false,
